#include <Wire.h>
#define PWM13       OCR4A // pin 13 with PWM modulation
//  SETUP I2C
byte DRV = 0x5A;    //DRV2605 slave address - uniq adress
byte ModeReg = 0x01;

#define BUTTON_PIN_MORSE 18 
#define BUTTON_PIN_MODE 19

char letters_and_numbers[36]={'A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z','0','1','2','3','4','5','6','7','8','9'};
String morse_code_strings[36]={".-","-...","-.-.","-..",".","..-.","--.","....","..",".---","-.-",".-..","--","-.","---",".--.","--.-",".-.","...","-","..-","...-",".--","-..-","-.--","--..",
"-----",".----","..---","...--","....-",".....","-....","--...","---..","----."};

String morse_string;
String buffer;

bool morse_mode;
const unsigned long dot_interval = 100;
const unsigned long dash_interval = 300;
const unsigned long letter_interval = 1000;
const unsigned long word_interval = 2000;

unsigned long current_millis;
unsigned long prev_millis;

int dot_dash_flag;
int letter_flag;
int word_flag;
int dot_dash_count;
int prev_state_mode = HIGH;

void setup() {
  Wire.begin(); // 
  Serial.begin(9600);
  Serial.setTimeout(1000); // read maximum timeout
  pwm13configure();
  delay(2);
  initializeDRV2605();  //initializes DRV2605 - driver

  pinMode(BUTTON_PIN_MORSE, INPUT_PULLUP);
  pinMode(BUTTON_PIN_MODE, INPUT_PULLUP);

  prev_state_mode = HIGH;
  morse_mode = false;
  dot_dash_flag = 0;
  letter_flag = 0;
  word_flag = 0;
  prev_millis = 0;
  morse_string = "";
  buffer = "";
  dot_dash_count = 0;
}


void loop() {

  int button_state_morse = digitalRead(BUTTON_PIN_MORSE);
  int button_state_mode = digitalRead(BUTTON_PIN_MODE);
  if(!morse_mode) { // TEXT MODE
    if (button_state_mode == LOW && prev_state_mode != button_state_mode) {
      morse_mode = true;
      Serial.println("MORSE MODE -> WAIT FOR MESSAGE");
      delay(100); 

    } else {
      if (Serial.available() > 0) { // if there is something to read - number of bites
        String str = Serial.readString();
        for (int i = 0; i < str.length(); i++) {
          char c = str.charAt(i);
          if (isWhitespace(c)){ // check space
            pause(word_interval);// Space between words is 7 dots duration. - I changed it to 20 times
          } else {
            fromCharToMorse(c);
          }
          //Serial.print(c); info for us
          pause(letter_interval); // Space between letters is 3 dots duration. - I changed it to 10 times to make it more clear
        }
        Serial.println();
      }
    }
  } else { // MORSE MODE
    if (button_state_mode == LOW && prev_state_mode != button_state_mode) {
      morse_mode = false; 
      Serial.print('\n'); 
      Serial.print("MESSAGE: "); 
      Serial.print(buffer);
      buffer = "";
      Serial.print('\n'); 
      Serial.println("TEXT MODE -> WRITE MESSAGE");
      delay(100); 
    } else {
      if (button_state_morse == LOW) {
        if (dot_dash_flag == 0) {
          prev_millis = millis();
          dot_dash_flag = 1;
        }
      } else {
        current_millis = millis();
        long diff = current_millis - prev_millis;
        if (word_flag == 1 && diff >= word_interval) { // WORD
          //Serial.print(' '); info for us
          buffer += " ";
          word_flag = 0;
          morse_string = "";
          letter_flag = 0;
          dot_dash_count = 0;
          prev_millis = millis();
          dot_dash_flag = 0;
        }
        if (letter_flag == 1 && (dot_dash_count == 5 || diff >= letter_interval)) { // LETTER
          char letter = fromMorseToChar(morse_string);
          //Serial.print(letter); info for us
          buffer += letter;
          morse_string = "";
          word_flag = 1;
          letter_flag = 0;
          dot_dash_count = 0;
          prev_millis = millis();
          dot_dash_flag = 0;
        }
        if (dot_dash_flag == 1) {
          if (diff >= dash_interval) {
            //Serial.print("-"); info for us
            morse_string += "-";
            dot_dash_count++;
          } else if (diff >= dot_interval) {
           // Serial.print("."); info for us
            morse_string += ".";
            dot_dash_count++;
          }

          letter_flag = 1;
          prev_millis = millis();
          dot_dash_flag = 0;
        }
      }
    }
  }
  delay(50);
  prev_state_mode = button_state_mode;
}

char fromMorseToChar(String morse_letter){
  for(int i = 0;i < sizeof(letters_and_numbers); i++){
    if(morse_letter == morse_code_strings[i]){
      return letters_and_numbers[i];
    }
  }
  return '*';
}


void fromCharToMorse(char c){
  for(int i = 0;i < sizeof(letters_and_numbers); i++) {
    if(c == letters_and_numbers[i]){
      for (int j = 0; j < morse_code_strings[i].length(); j++) {
        char morse = morse_code_strings[i].charAt(j);
        //Serial.print(morse); info for us
        if(morse == '.'){
          pulse(0.8,100);
        } else if (morse == '-'){
          vibrate(130,0.4,0.3,41);
        }
        pause(100);
      }
    }
  }
}



/*
pulse(double intensity, double milliseconds)
pulse drives the hammer towards the closed end of the TacHammer
intensity defines the strength of the pulse. It ranges from [0-1] with 1 being strongest
When the hammer rebounds off of the repelling magnetic array, the inaudible pulse haptic
sensation is created milliseconds is the length of time the coil is activated in ms pulse is
intended to be sequenced with subsequent pulse and hit commands and if called on its own,
the hammer may travel after the rebound and strike the open end 
*/

void pulse(double intensity, double milliseconds) 
{
  int minimumint = 140;
  int maximumint = 255;
  int pwmintensity = (intensity * (maximumint - minimumint)) + minimumint;
  standbyOffB();
  PWM13 = pwmintensity;
  usdelay(milliseconds);
  standbyOnB();
}

void pause(double milliseconds) 
{
  double us = milliseconds - ((int)milliseconds);
  standbyOnB();
  for (int i = 0; i <= milliseconds; i++) 
  {
    delay(1);
  }
  delayMicroseconds(us * 1000);
}

void usdelay(double time) 
{
  double us = time - ((int)time);
  for (int i = 0; i <= time; i++) 
  {
    delay(1);
  }
  delayMicroseconds(us * 1000);
}

/*
 vibrate (double frequency, doub;le intensity, double duration, int dutycyle)
Vibrate repeatedly calls the pulse command to druve the TecHammer into the closed end of
the Techammer.
frequency defines the frequency of the vibration
intensity defines the strength of the vibrate. In ranges from 0 to 1, with 1 being the strongest.
duration defines the length of the vibration in s
dutycycle defines what percent of the perdio the TecHammer is active, it ranges from 0 to 100
Suggested duty cycles for vibrate
FREQ| 10| 30| 50| 70| 90|110|130|150|170|190|210|230|250|270|290
DUTY| 32| 28| 33| 36| 38| 37| 41| 47| 45| 47| 39| 40| 44| 43| 40
*/
void vibrate(double frequency, double intensity, double duration, int dutycycle) 
{
  int max_hit = 12;
  int min_hit = 1;
  int crossover = 60;
  int hitduration = 10 * dutycycle / frequency;

  boolean hold = false;
  double delayy;
  delayy = (1 / frequency * 1000) - hitduration;
  double timedown;
  timedown = duration * 1000;

  if (duration == 0)
  {
    hold =  true;
  }

  while (hold)
  {
    pulse(intensity, hitduration);
    pause(delayy);
  }

  while (timedown >= 0 && frequency < crossover)
  {
    pulse(intensity, hitduration);
    pause(3);
    pulse(0.002, delayy-3);

    timedown -= (delayy + hitduration);
  }

  while (timedown >= 0 && frequency >= crossover) 
  {
    pulse(intensity, hitduration);
    pause(delayy);
    timedown -= (delayy + hitduration);
  }
}


void standbyOnB() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);            // sets register pointer to the mode register (0x01)
  Wire.write(0x43);               // Puts the device pwm mode
  Wire.endTransmission();
}

void standbyOffB() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);               // sets register pointer to the mode register (0x01)
  Wire.write(0x03);               // Sets Waveform Mode to pwm
  Wire.endTransmission();
}

void initializeDRV2605() 
{
  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);            // sets register pointer to the mode register (0x01)
  Wire.write(0x00);               // clear standby
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x1D);              // sets register pointer to the Libarary Selection register (0x1D)
  Wire.write(0xA8);              // set RTP unsigned
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x03);
  Wire.write(0x02);              // set to Library B, most aggresive
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(0x17);               // sets full scale reference
  Wire.write(0xff);               //
  Wire.endTransmission();

  Wire.beginTransmission(DRV);
  Wire.write(ModeReg);               // sets register pointer to the mode register (0x01)
  Wire.write(0x03);               // Sets Mode to pwm
  Wire.endTransmission();

  delay(100);

}


// Configure the PWM clock
#define PWM12k  5   //  11719 Hz

void pwm13configure() {
  // TCCR4A configuration
  TCCR4A = 0;

  // TCCR4B configuration
  TCCR4B = PWM12k;

  // TCCR4C configuration
  TCCR4C = 0;

  // TCCR4D configuration
  TCCR4D = 0;

  // TCCR4D configuration
  TCCR4D = 0;

  // PLL Configuration
  // Use 96MHz / 2 = 48MHz
  PLLFRQ = (PLLFRQ & 0xCF) | 0x30;
  // PLLFRQ=(PLLFRQ&0xCF)|0x10; // Will double all frequencies

  // Terminal count for Timer 4 PWM
  OCR4C = 255;

  //pwmSet6();
  pwmSet13();
}


// Set PWM to D13 (Timer4 A)
void pwmSet13() {
  OCR4A = 0;  
  DDRC |= _BV(7); 
  TCCR4A = 0x82; 
}


